@@ -20,6 +20,7 @@ enum fio_file_flags {
 	FIO_FILE_done		= 1 << 3,	/* io completed to this file */
 	FIO_FILE_size_known	= 1 << 4,	/* size has been set */
 	FIO_FILE_hashed		= 1 << 5,	/* file is on hash */
+	FIO_FILE_partial_mmap	= 1 << 6,	/* can't do full mmap */
 };
 
 enum file_lock_mode {
@@ -73,6 +74,11 @@ struct fio_file {
 
 	unsigned long long last_pos;
 
+	/*
+	 * For use by the io engine
+	 */
+	unsigned long long file_pos;
+
 	/*
 	 * if io is protected by a semaphore, this is set
 	 */
@@ -114,6 +120,7 @@ FILE_FLAG_FNS(extend);
 FILE_FLAG_FNS(done);
 FILE_FLAG_FNS(size_known);
 FILE_FLAG_FNS(hashed);
+FILE_FLAG_FNS(partial_mmap);
 #undef FILE_FLAG_FNS
 
 /*
@@ -131,6 +138,7 @@ extern int __must_check pre_read_files(struct thread_data *);
 extern int add_file(struct thread_data *, const char *);
 extern void get_file(struct fio_file *);
 extern int __must_check put_file(struct thread_data *, struct fio_file *);
+extern void put_file_log(struct thread_data *, struct fio_file *);
 extern void lock_file(struct thread_data *, struct fio_file *, enum fio_ddir);
 extern void unlock_file(struct thread_data *, struct fio_file *);
 extern void unlock_file_all(struct thread_data *, struct fio_file *);
@@ -144,6 +152,7 @@ static inline void fio_file_reset(struct fio_file *f)
 {
 	f->last_free_lookup = 0;
 	f->last_pos = f->file_offset;
+	f->file_pos = -1ULL;
 	if (f->file_map)
 		memset(f->file_map, 0, f->num_maps * sizeof(int));
 }
