@@ -39,6 +39,7 @@
 #include "smalloc.h"
 #include "verify.h"
 #include "diskutil.h"
+#include "cgroup.h"
 
 unsigned long page_mask;
 unsigned long page_size;
@@ -60,6 +61,8 @@ static volatile int fio_abort;
 static int exit_value;
 static struct itimerval itimer;
 static pthread_t gtod_thread;
+static struct flist_head *cgroup_list;
+static char *cgroup_mnt;
 
 struct io_log *agg_io_log[2];
 
@@ -1038,6 +1041,15 @@ static void *thread_main(void *data)
 	 */
 	fio_mutex_remove(td->mutex);
 
+	if (td->o.uid != -1U && setuid(td->o.uid)) {
+		td_verror(td, errno, "setuid");
+		goto err;
+	}
+	if (td->o.gid != -1U && setgid(td->o.gid)) {
+		td_verror(td, errno, "setgid");
+		goto err;
+	}
+
 	/*
 	 * May alter parameters that init_io_u() will use, so we need to
 	 * do this first.
@@ -1075,6 +1087,9 @@ static void *thread_main(void *data)
 		}
 	}
 
+	if (td->o.cgroup_weight && cgroup_setup(td, cgroup_list, &cgroup_mnt))
+		goto err;
+
 	if (nice(td->o.nice) == -1) {
 		td_verror(td, errno, "nice");
 		goto err;
@@ -1204,6 +1219,7 @@ err:
 	close_and_free_files(td);
 	close_ioengine(td);
 	cleanup_io_u(td);
+	cgroup_shutdown(td, &cgroup_mnt);
 
 	if (td->o.cpumask_set) {
 		int ret = fio_cpuset_exit(&td->o.cpumask);
@@ -1652,6 +1668,9 @@ int main(int argc, char *argv[])
 
 	status_timer_arm();
 
+	cgroup_list = smalloc(sizeof(*cgroup_list));
+	INIT_FLIST_HEAD(cgroup_list);
+
 	run_threads();
 
 	if (!fio_abort) {
@@ -1663,6 +1682,11 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	cgroup_kill(cgroup_list);
+	sfree(cgroup_list);
+	if (cgroup_mnt)
+		sfree(cgroup_mnt);
+
 	fio_mutex_remove(startup_mutex);
 	fio_mutex_remove(writeout_mutex);
 	return exit_value;
