@@ -1081,6 +1081,15 @@ static struct fio_option options[] = {
 		},
 		.parent = "nrfiles",
 	},
+#ifdef FIO_HAVE_FALLOCATE
+	{
+		.name	= "fallocate",
+		.type	= FIO_OPT_BOOL,
+		.off1	= td_var_offset(fallocate),
+		.help	= "Use fallocate() when laying out files",
+		.def	= "1",
+	},
+#endif
 	{
 		.name	= "fadvise_hint",
 		.type	= FIO_OPT_BOOL,
@@ -1809,16 +1818,15 @@ static struct fio_keyword fio_keywords[] = {
 
 void fio_keywords_init(void)
 {
-	unsigned long mb_memory;
+	unsigned long long mb_memory;
 	char buf[128];
 	long l;
 
 	sprintf(buf, "%lu", page_size);
 	fio_keywords[0].replace = strdup(buf);
 
-	l = sysconf(_SC_PHYS_PAGES);
-	mb_memory = l * (page_size / 1024UL);
-	sprintf(buf, "%lu", mb_memory);
+	mb_memory = os_phys_mem() / page_size;
+	sprintf(buf, "%llu", mb_memory);
 	fio_keywords[1].replace = strdup(buf);
 
 	l = sysconf(_SC_NPROCESSORS_ONLN);
